@@ -1,4 +1,74 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
+/*
+ * anjuta-shell.c
+ * Copyright (C) Naba Kumar  <naba@gnome.org>
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+/**
+ * SECTION:anjuta-shell
+ * @title: AnjutaShell
+ * @short_description: Application shell interface
+ * @see_also:
+ * @stability: Unstable
+ * @include: libanjuta/anjuta-shell.h
+ * 
+ * Shell is the playground where plugins are loaded and their UI
+ * widgets shown. It is also a place where plugins export objects for letting
+ * other pluings to use. Plugins are loaded into shell on demand, but some
+ * plugins are loaded on startup (such as help and text editor plugin).
+ * Demand to load a plugin can be made by requesting for a primary inferface
+ * using anjuta_shell_get_interface() or anjuta_shell_get_object().
+ * 
+ * Plugins can add widgets in shell with
+ * anjuta_shell_add_widget() and remove with anjuta_shell_remove_widget()
+ * functions.
+ * 
+ * In Anjuta, shell is implemented using an advanced widget docking system,
+ * allowing plugin widgets to dock, undock and layout in any fashion. Dock
+ * layout is also maintained internally and is transparent to plugin
+ * implementations.
+ * 
+ * #AnjutaShell allows plugins to export arbitrary objects as <emphasis>
+ * values</emphasis> in its <emphasis>Values System</emphasis>. "value_added"
+ * and "value_removed" signals are emitted when a value is added to or
+ * removed from the <emphasis>Values System</emphasis>, hence notifying
+ * plugins of its state. However, plugins should really not connect directly
+ * to these signals, because they are emitted for all values
+ * and not just for the values the plugin is interested in. Instead,
+ * to monitor specific <emphasis>Values</emphasis>, plugins should
+ * setup watches using anjuta_plugin_add_watch().
+ * 
+ * <emphasis>Values</emphasis> are added, get or removed with
+ * anjuta_shell_add_value() and anjuta_shell_get_value() or
+ * anjuta_shell_remove_value(). There multi-valued equivalent functions
+ * can be used to manipulate multiple values at once.
+ * 
+ * <emphasis>Values</emphasis> are identified with names. Since <emphasis>
+ * Values</emphasis> are effectively variables, their names should follow
+ * the standard GNOME variable naming convention and should be as descriptive
+ * as possible (e.g project_root_directory, project_name etc.). It is also
+ * essential that meaningful prefix be given to names so that <emphasis>
+ * Values</emphasis> are easily grouped (e.g all values exported by a 
+ * project manager should start with project_ prefix).
+ * 
+ * Plugins can find other plugins with anjuta_shell_get_object() or 
+ * anjuta_shell_get_interface() based on their primary interfaces.
+ */
+
 #include <config.h>
 #include <string.h>
 #include <gobject/gvaluecollector.h>
@@ -12,6 +82,7 @@ typedef struct {
 	gchar *title;
 	gchar *stock_id;
 	AnjutaShellPlacement placement;
+	gboolean locked;
 } WidgetQueueData;
 
 static void
@@ -27,11 +98,12 @@ on_widget_data_free (WidgetQueueData *data)
 static void
 on_widget_data_add (WidgetQueueData *data, AnjutaShell *shell)
 {
-	ANJUTA_SHELL_GET_IFACE (shell)->add_widget (shell, data->widget,
+	ANJUTA_SHELL_GET_IFACE (shell)->add_widget_full (shell, data->widget,
 												data->name,
 												data->title,
 												data->stock_id,
 												data->placement,
+												data->locked,
 												NULL);
 }
 
@@ -67,7 +139,7 @@ anjuta_shell_error_quark (void)
  * 1 and real thawing happens when the count reaches 0.
  */
 void
-anjuta_shell_freeze (AnjutaShell *shell, GError *error)
+anjuta_shell_freeze (AnjutaShell *shell, GError **error)
 {
 	gint freeze_count;
 	
@@ -88,7 +160,7 @@ anjuta_shell_freeze (AnjutaShell *shell, GError *error)
  * when the count reaches 0.
  */
 void
-anjuta_shell_thaw (AnjutaShell *shell, GError *error)
+anjuta_shell_thaw (AnjutaShell *shell, GError **error)
 {
 	gint freeze_count;
 	
@@ -104,7 +176,7 @@ anjuta_shell_thaw (AnjutaShell *shell, GError *error)
 	if (freeze_count <= 0)
 	{
 		/* Add all pending widgets */
-		DEBUG_PRINT ("Thawing shell ...");
+		/* DEBUG_PRINT ("%s", "Thawing shell ..."); */
 		
 		GQueue *queue;
 		queue = g_object_get_data (G_OBJECT (shell), "__widget_queue");
@@ -141,6 +213,41 @@ anjuta_shell_add_widget (AnjutaShell *shell,
 			 const char *stock_id,
 			 AnjutaShellPlacement placement,
 			 GError **error)
+{
+	anjuta_shell_add_widget_full(shell, widget, name, title,
+							stock_id, placement, FALSE, error);
+}
+
+/**
+ * anjuta_shell_add_widget_full:
+ * @shell: A #AnjutaShell interface.
+ * @widget: Then widget to add
+ * @name: Name of the widget. None translated string used to identify it in 
+ * the shell.
+ * @stock_id: Icon stock ID. Could be null.
+ * @title: Translated string which is displayed along side the widget when
+ * required (eg. as window title or notebook tab label).
+ * @placement: Placement of the widget in shell.
+ * @locked: Whether to lock that widget (do not use this, it's only
+ * 			useful to some stock plugins
+ * @error: Error propagation object.
+ *
+ * Adds @widget in the shell. The @placement tells where the widget should
+ * appear, but generally it will be overridden by the container
+ * (dock, notebook, GtkContainer etc.) saved layout.
+ *
+ * Normally just use anjuta_shell_add_widget() because you do not
+ * use locking.
+ */
+void
+anjuta_shell_add_widget_full (AnjutaShell *shell,
+			 GtkWidget *widget,
+			 const char *name,
+			 const char *title,
+			 const char *stock_id,
+			 AnjutaShellPlacement placement,
+			 gboolean locked,
+			 GError **error)
 {
 	GQueue *widget_queue;
 	gint freeze_count;
@@ -156,9 +263,9 @@ anjuta_shell_add_widget (AnjutaShell *shell,
 													   "__freeze_count"));
 	if (freeze_count <= 0)
 	{
-		ANJUTA_SHELL_GET_IFACE (shell)->add_widget (shell, widget, name,
+		ANJUTA_SHELL_GET_IFACE (shell)->add_widget_full (shell, widget, name,
 													title, stock_id,
-													placement, error);
+													placement, locked, error);
 	}
 	else
 	{
@@ -177,6 +284,7 @@ anjuta_shell_add_widget (AnjutaShell *shell,
 		qd->widget = widget;
 		qd->name = g_strdup (name);
 		qd->title = g_strdup (title);
+		qd->locked = locked;
 		if (stock_id)
 			qd->stock_id = g_strdup (stock_id);
 		qd->placement = placement;
@@ -610,6 +718,42 @@ anjuta_shell_get_preferences (AnjutaShell *shell, GError **error)
 	return ANJUTA_SHELL_GET_IFACE (shell)->get_preferences (shell, error);
 }
 
+/**
+ * anjuta_shell_get_plugin_manager:
+ * @shell: A #AnjutaShell interface
+ * @error: Error propagation object
+ *
+ * Retrieves the #AnjutaPluginManager object associated with the shell.
+ *
+ * Return value: The #AnjutaPluginManager object.
+ */
+AnjutaPluginManager*
+anjuta_shell_get_plugin_manager (AnjutaShell *shell, GError **error)
+{
+	g_return_val_if_fail (shell != NULL, NULL);
+	g_return_val_if_fail (ANJUTA_IS_SHELL (shell), NULL);
+
+	return ANJUTA_SHELL_GET_IFACE (shell)->get_plugin_manager (shell, error);
+}
+
+/**
+ * anjuta_shell_get_profile_manager:
+ * @shell: A #AnjutaShell interface
+ * @error: Error propagation object
+ *
+ * Retrieves the #AnjutaProfileManager object associated with the shell.
+ *
+ * Return value: The #AnjutaProfileManager object.
+ */
+AnjutaProfileManager*
+anjuta_shell_get_profile_manager (AnjutaShell *shell, GError **error)
+{
+	g_return_val_if_fail (shell != NULL, NULL);
+	g_return_val_if_fail (ANJUTA_IS_SHELL (shell), NULL);
+
+	return ANJUTA_SHELL_GET_IFACE (shell)->get_profile_manager (shell, error);
+}
+
 void
 anjuta_shell_session_save (AnjutaShell *shell, const gchar *session_directory,
 						   GError **error)
@@ -643,23 +787,45 @@ anjuta_shell_session_load (AnjutaShell *shell, const gchar *session_directory,
 	/* Do not allow multiple session loadings at once. Could be a trouble */
 	if (g_object_get_data (G_OBJECT (shell), "__session_loading"))
 	{
-		DEBUG_PRINT ("A session load is requested in the middle of another!!");
+		/* DEBUG_PRINT ("%s", "A session load is requested in the middle of another!!"); */
 		return;
 	}
 	g_object_set_data (G_OBJECT (shell), "__session_loading", "1");
 
 	session = anjuta_session_new (session_directory);
+	g_signal_emit_by_name (G_OBJECT (shell), "load_session",
+						   ANJUTA_SESSION_PHASE_START, session);
 	g_signal_emit_by_name (G_OBJECT (shell), "load_session",
 						   ANJUTA_SESSION_PHASE_FIRST, session);
 	g_signal_emit_by_name (G_OBJECT (shell), "load_session",
 						   ANJUTA_SESSION_PHASE_NORMAL, session);
 	g_signal_emit_by_name (G_OBJECT (shell), "load_session",
 						   ANJUTA_SESSION_PHASE_LAST, session);
+	g_signal_emit_by_name (G_OBJECT (shell), "load_session",
+						   ANJUTA_SESSION_PHASE_END, session);
 	g_object_unref (session);
 
 	g_object_set_data (G_OBJECT (shell), "__session_loading", NULL);
 }
 
+void
+anjuta_shell_save_prompt (AnjutaShell *shell,
+						  AnjutaSavePrompt *save_prompt,
+						  GError **error)
+{
+	g_return_if_fail (ANJUTA_IS_SHELL (shell));
+	g_return_if_fail (ANJUTA_IS_SAVE_PROMPT (save_prompt));
+	g_signal_emit_by_name (shell, "save-prompt", save_prompt);
+}
+
+void
+anjuta_shell_notify_exit (AnjutaShell *shell,
+						  GError **error)
+{
+	g_return_if_fail (ANJUTA_IS_SHELL (shell));
+	g_signal_emit_by_name (shell, "exiting");
+}
+
 static void
 anjuta_shell_base_init (gpointer gclass)
 {
@@ -701,6 +867,21 @@ anjuta_shell_base_init (gpointer gclass)
 			      G_TYPE_NONE, 2,
 				  G_TYPE_INT,
 			      G_TYPE_OBJECT);
+		g_signal_new ("save-prompt",
+			      ANJUTA_TYPE_SHELL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (AnjutaShellIface, save_prompt),
+			      NULL, NULL,
+			      anjuta_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
+		g_signal_new ("exiting",
+			      ANJUTA_TYPE_SHELL,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (AnjutaShellIface, exiting),
+			      NULL, NULL,
+			      anjuta_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 		initialized = TRUE;
 	}
 }
