@@ -55,73 +55,99 @@
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #define FILE_BUFFER_SIZE 1024
 
-gboolean
-anjuta_util_copy_file (gchar * src, gchar * dest, gboolean show_error)
+static void
+anjuta_util_from_file_to_file (GInputStream *istream,
+							   GOutputStream *ostream)
 {
-	FILE *input_fp, *output_fp;
+	gsize bytes = 1;
+	GError *error = NULL;
 	gchar buffer[FILE_BUFFER_SIZE];
-	gint bytes_read, bytes_written;
-	gboolean error;
 	
-	error = TRUE;
-	
-	input_fp = fopen (src, "rb");
-	if (input_fp == NULL)
+	while (bytes != 0 && bytes != -1)
 	{
-		if( show_error)
-			anjuta_util_dialog_error_system (NULL, errno,
-											 _("Unable to read file: %s."),
-											 src);
-		return FALSE;
-	}
-	
-	output_fp = fopen (dest, "wb");
-	if (output_fp == NULL)
-	{
-		if( show_error)
-			anjuta_util_dialog_error_system (NULL, errno,
-											 _("Unable to create file: %s."),
-											 dest);
-		fclose (input_fp);
-		return TRUE;
-	}
-	
-	for (;;)
-	{
-		bytes_read = fread (buffer, 1, FILE_BUFFER_SIZE, input_fp);
-		if (bytes_read != FILE_BUFFER_SIZE && ferror (input_fp))
-		{
-			error = FALSE;
+		bytes = g_input_stream_read (istream, buffer,
+					     sizeof (buffer),
+					     NULL, &error);
+		if (error)
 			break;
-		}
-		
-		if (bytes_read)
-		{
-			bytes_written = fwrite (buffer, 1, bytes_read, output_fp);
-			if (bytes_read != bytes_written)
-			{
-				error = FALSE;
-				break;
-			}
-		}
-		
-		if (bytes_read != FILE_BUFFER_SIZE && feof (input_fp))
-		{
+
+		g_output_stream_write (ostream, buffer,
+				       bytes,
+				       NULL, &error);
+		if (error)
 			break;
-		}
 	}
 	
-	fclose (input_fp);
-	fclose (output_fp);
+	if (error)
+	{
+		g_warning (error->message);
+		g_error_free (error);
+		error = NULL;
+	}
 	
-	if( show_error && (error == FALSE))
-		anjuta_util_dialog_error_system (NULL, errno,
-										 _("Unable to complete file copy"));
-	return error;
+	if (!g_output_stream_close (ostream, NULL, &error))
+	{
+		g_warning (error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+	if (!g_input_stream_close (istream, NULL, &error))
+	{
+		g_warning (error->message);
+		g_error_free (error);
+	}
+}
+
+/**
+ * anjuta_util_copy_file:
+ * @src: the file where copy
+ * @dest: the path to copy the @src
+ * @show_error: TRUE to show a dialog error
+ *
+ * Copies @src to @dest and shows a dialog error in case is needed.
+ *
+ * Returns: TRUE if there was an error copying the file.
+ */
+gboolean
+anjuta_util_copy_file (const gchar * src, const gchar * dest, gboolean show_error)
+{
+	GFile *src_file, *dest_file;
+	GFileInputStream *istream;
+	GFileOutputStream *ostream;
+	GError *error = NULL;
+	gboolean toret = FALSE;
+	
+	src_file = g_file_new_for_path (src);
+	dest_file = g_file_new_for_path (dest);
+	
+	istream = g_file_read (src_file, NULL, &error);
+	if (error)
+		goto free;
+		
+	ostream = g_file_create (dest_file, G_FILE_CREATE_NONE,
+							 NULL, &error);
+	if (error)
+		goto free;
+	
+	anjuta_util_from_file_to_file (G_INPUT_STREAM (istream), G_OUTPUT_STREAM (ostream));
+	
+free: if (error)
+	{
+		if (show_error)
+			anjuta_util_dialog_error_system (NULL, error->code,
+											 error->message);
+	
+		g_warning ("%s", error->message);
+		
+		toret = TRUE;
+	}
+	
+	g_object_unref (src_file);
+	g_object_unref (dest_file);
+	
+	return toret;
 }
 
 void
@@ -248,7 +274,7 @@ anjuta_util_dialog_error (GtkWindow *parent, const gchar *mesg, ...)
 	dialog = gtk_message_dialog_new (real_parent,
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_ERROR,
-									 GTK_BUTTONS_CLOSE, message);
+									 GTK_BUTTONS_CLOSE, "%s", message);
 	g_signal_connect (G_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (dialog);
@@ -280,7 +306,7 @@ anjuta_util_dialog_warning (GtkWindow *parent, const gchar * mesg, ...)
 	dialog = gtk_message_dialog_new (real_parent,
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_WARNING,
-									 GTK_BUTTONS_CLOSE, message);
+									 GTK_BUTTONS_CLOSE, "%s", message);
 	g_signal_connect (G_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (dialog);
@@ -311,7 +337,7 @@ anjuta_util_dialog_info (GtkWindow *parent, const gchar * mesg, ...)
 	dialog = gtk_message_dialog_new (real_parent,
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_INFO,
-									 GTK_BUTTONS_CLOSE, message);
+									 GTK_BUTTONS_CLOSE, "%s", message);
 	g_signal_connect (G_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (dialog);
@@ -333,7 +359,8 @@ anjuta_util_dialog_error_system (GtkWindow* parent, gint errnum,
 	va_end (args);
 
 	if (0 != errnum) {
-		tot_mesg = g_strconcat (message, _("\nSystem: "),
+		/* Avoid space in translated string */
+		tot_mesg = g_strconcat (message, "\n", _("System:"), " ",
 								g_strerror(errnum), NULL);
 		g_free (message);
 	} else
@@ -351,7 +378,7 @@ anjuta_util_dialog_error_system (GtkWindow* parent, gint errnum,
 	dialog = gtk_message_dialog_new (real_parent,
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_ERROR,
-									 GTK_BUTTONS_CLOSE, tot_mesg);
+									 GTK_BUTTONS_CLOSE, "%s", tot_mesg);
 	g_signal_connect (G_OBJECT (dialog), "response",
 					  G_CALLBACK (gtk_widget_destroy), NULL);
 	gtk_widget_show (dialog);
@@ -383,7 +410,7 @@ anjuta_util_dialog_boolean_question (GtkWindow *parent, const gchar *mesg, ...)
 	dialog = gtk_message_dialog_new (real_parent,
 									 GTK_DIALOG_DESTROY_WITH_PARENT,
 									 GTK_MESSAGE_QUESTION,
-									 GTK_BUTTONS_YES_NO, message);
+									 GTK_BUTTONS_YES_NO, "%s", message);
 
 	ret = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
@@ -897,6 +924,7 @@ anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 {
 	pid_t pid;
 	gchar *shell;
+	gint err;
 	
 	g_return_val_if_fail (command != NULL, -1);
 	
@@ -907,7 +935,7 @@ anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 		if(dir)
 		{
 			anjuta_util_create_dir (dir);
-			chdir (dir);
+			err = chdir (dir);
 		}
 		execlp (shell, shell, "-c", command, NULL);
 		g_warning (_("Cannot execute command: %s (using shell %s)\n"), command, shell);
@@ -1048,105 +1076,173 @@ anjuta_util_escape_quotes(const gchar* str)
 	return buffer;
 }
 
-/* FIXME: Use gio instead */
 /* Diff the text contained in uri with text. Return true if files
-differ, FALSE if they are identical.
-FIXME: Find a better algorithm, this seems ineffective */
+differ, FALSE if they are identical.*/
 
 gboolean anjuta_util_diff(const gchar* uri, const gchar* text)
 {
-	GnomeVFSFileSize bytes_read;
-	gchar* file_text;
-	GnomeVFSFileInfo info;
-	GnomeVFSHandle* handle = NULL;
-	
-	gnome_vfs_get_file_info(uri, &info, GNOME_VFS_FILE_INFO_DEFAULT);
-	
-	if (info.size == 0 && text == NULL)
+	GFile *file;
+	GFileInfo *file_info;
+	guint64 size;
+	gchar* file_text = NULL;
+	gsize bytes_read;
+
+	file = g_file_new_for_uri (uri);
+	file_info = g_file_query_info (file,
+			G_FILE_ATTRIBUTE_STANDARD_SIZE,
+			G_FILE_QUERY_INFO_NONE,
+			NULL,
+			NULL);
+
+	if (file_info == NULL)
+	{
+		g_object_unref (file);
+		return TRUE;
+	}
+
+	size = g_file_info_get_attribute_uint64(file_info,
+			G_FILE_ATTRIBUTE_STANDARD_SIZE);
+	g_object_unref (file_info);
+
+	if (size == 0 && text == NULL)
+	{
+		g_object_unref (file);
 		return FALSE;
-	else if (info.size == 0 || text == NULL)
-		return TRUE;
-	
-	file_text = g_new0(gchar, info.size + 1);
-	
-	if (gnome_vfs_open(&handle, uri, GNOME_VFS_OPEN_READ != GNOME_VFS_OK))
-		return TRUE;
-	
-	if ((gnome_vfs_read(handle, file_text, info.size, &bytes_read) == GNOME_VFS_OK)
-		&& (bytes_read == info.size))
+	}
+	else if (size == 0 || text == NULL)
 	{
-		gnome_vfs_close(handle);
-		
-		if ((g_utf8_strlen(file_text, -1) == g_utf8_strlen(text, -1))
-			&& strcmp(file_text, text) == 0)
-			return FALSE;
+		g_object_unref (file);
 		return TRUE;
 	}
-	else
+
+	if (!g_file_load_contents(file, 
+				NULL,
+				&file_text, 
+				&bytes_read,
+				NULL,
+				NULL))
 	{
-		gnome_vfs_close(handle);
+		g_object_unref (file);
 		return TRUE;
 	}
+	g_object_unref (file);
+
+	if (bytes_read != size)
+	{
+		g_free (file_text);
+		return TRUE;
+	}
+
+	/* according to g_file_load_contents's documentation
+	 * file_text is guaranteed to end with \0.
+	 */
+	if (strcmp (file_text, text) == 0)
+	{
+		g_free (file_text);
+		return FALSE;
+	}
+
+	g_free (file_text);
+	return TRUE;
 }
 
+/**
+ * anjuta_util_is_project_file:
+ * @filename: the file name
+ *
+ * Return TRUE if the file is an anjuta project file. It is implemented by
+ * checking only the file extension. So it does not check the existence
+ * of the file. But it is working on an URI if it does not containt a
+ * fragment.
+ *
+ * Returns: TRUE if the file is a project file, else FALSE
+ */
 gboolean
-anjuta_util_path_has_extension (const gchar *path, const gchar *ext)
+anjuta_util_is_project_file (const gchar *filename)
 {
-	if (strlen (path) <= strlen (ext))
-		return FALSE;
-	if ((path[strlen (path) - strlen (ext) - 1] == '.') &&
-		(strcmp (&path[strlen (path) - strlen (ext)], ext) == 0))
-		return TRUE;
-	return FALSE;
+	gsize len = strlen (filename);
+	return (len > 8) && (strcmp (filename + len - 7, ".anjuta") == 0);
+}
+
+/**
+ * anjuta_util_get_file_mine_type:
+ * @file: the file
+ *
+ * Check if a file exists and return its mime type.
+ *
+ * Returns: NULL if the corresponding file doesn't exist or the mime type as a newly
+ * allocated string that must be freed with g_free().
+ */
+gchar *
+anjuta_util_get_file_mime_type (GFile *file)
+{
+	GFileInfo *info;
+	gchar *mime_type = NULL;
+	
+	g_return_val_if_fail (file != NULL, NULL);
+	
+	/* Get file information, check that the file exist at the same time */
+	info = g_file_query_info (file,
+										  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+										  G_FILE_QUERY_INFO_NONE,
+										  NULL,
+										  NULL);
+	
+	if (info != NULL)
+	{
+		const gchar *extension;
+		gchar *name;
+		
+		/* If Anjuta is not installed in system gnome prefix, the mime types 
+		 * may not have been correctly registed. In that case, we use the
+	 	 * following mime detection
+	 	 */
+		name = g_file_get_basename (file);
+		extension = strrchr(name, '.');
+		if (extension != NULL)
+		{
+			const static struct {gchar *extension; gchar *type;} anjuta_types[] = {
+									{"anjuta", "application/x-anjuta"},
+									{"prj", "application/x-anjuta-old"},
+									{"ui", "text/xml"},
+									{"glade", "application/x-glade"},
+									{NULL, NULL}};
+			gint i;
+				
+			for (i = 0; anjuta_types[i].extension != NULL; i++)
+			{
+				if (strcmp(extension + 1, anjuta_types[i].extension) == 0)
+				{
+					mime_type = g_strdup (anjuta_types[i].type);
+					break;
+				}
+			}
+		}
+		g_free (name);
+	
+		/* Use mime database if it is not an Anjuta type */
+		if (mime_type == NULL)
+		{
+			mime_type = g_content_type_get_mime_type (g_file_info_get_content_type(info));
+		}
+		
+		g_object_unref (info);
+	}
+
+	return mime_type;
 }
 
 gchar *
-anjuta_util_get_uri_mime_type (const gchar *uri)
+anjuta_util_get_local_path_from_uri (const gchar *uri)
 {
-	GnomeVFSURI *vfs_uri;
-	const gchar *path;
-	gchar *mime_type;
-	
-	g_return_val_if_fail (uri != NULL, NULL);
-	
-	vfs_uri = gnome_vfs_uri_new (uri);
-	if (vfs_uri)
-		path = gnome_vfs_uri_get_path (vfs_uri);
-	else
-		path = NULL;
-	
-	/* If Anjuta is not installed in system gnome prefix, the mime types 
-	 * may not have been correctly registed. In that case, we use the
-	 * following mime detection
-	 */
-	if (!path)
-	{
-		mime_type = gnome_vfs_get_slow_mime_type (uri);
-	}
-	else if (anjuta_util_path_has_extension (path, "anjuta"))
-	{
-		mime_type = g_strdup ("application/x-anjuta");
-	}
-	else if (anjuta_util_path_has_extension (path, "prj"))
-	{
-		mime_type = g_strdup ("application/x-anjuta-old");
-	}
-	else if (anjuta_util_path_has_extension (path, "ui"))
-	{
-		mime_type = g_strdup ("text/xml");
-	}
-	else if (anjuta_util_path_has_extension (path, "glade"))
-	{
-		mime_type = g_strdup ("application/x-glade");
-	}
-	else
-	{
-		mime_type = gnome_vfs_get_slow_mime_type (uri);
-	}
-	
-	if (vfs_uri)
-		gnome_vfs_uri_unref (vfs_uri);
-	return mime_type;
+	GFile *file;
+	gchar *local_path;
+
+	file = g_file_new_for_uri (uri);
+	local_path = g_file_get_path (file);
+	g_object_unref (file);
+
+	return local_path;
 }
 
 #ifndef HAVE_LIBUTIL
@@ -1444,7 +1540,7 @@ int scandir(const char *dir, struct dirent ***namelist,
 #endif /* HAVE_LIBUTIL */
 
 void
-anjuta_util_help_display (GtkWindow   *parent,
+anjuta_util_help_display (GtkWidget   *parent,
 						  const gchar *doc_id,
 						  const gchar *file_name)
 {
@@ -1478,7 +1574,7 @@ anjuta_util_help_display (GtkWindow   *parent,
 
 	if (uri == NULL)
 	{
-		anjuta_util_dialog_error (parent, _("Unable to display help. Please make sure Anjuta "
+		anjuta_util_dialog_error (GTK_WINDOW (parent), _("Unable to display help. Please make sure Anjuta "
 								  "documentation package is install. It can be downloaded "
 								  "from http://anjuta.org"));
 
@@ -1573,6 +1669,68 @@ anjuta_util_replace_home_dir_with_tilde (const gchar *uri)
 	return g_strdup (uri);
 }
 
+/**
+ * anjuta_util_shell_expand:
+ * @string: input string
+ *
+ * Expand environment variables $(var_name) and tilde (~) in the input string.
+ *
+ * Returns: a newly-allocated string that must be freed with g_free().
+ */
+gchar*
+anjuta_util_shell_expand (const gchar *string)
+{
+	GString* expand;
+	
+	if (string == NULL) return NULL;
+	
+	expand = g_string_sized_new (strlen (string));
+	
+	for (; *string != '\0'; string++)
+	{
+		switch (*string)
+		{
+			case '$':
+			{
+				/* Variable expansion */
+				const gchar *end;
+				gint var_name_len;
+
+				end = string + 1;
+				while (isalnum (*end) || (*end == '_')) end++;
+				var_name_len = end - string - 1;
+				if (var_name_len > 0)
+				{
+					const gchar *value;
+					
+					g_string_append_len (expand, string + 1, var_name_len);
+					value = g_getenv (expand->str + expand->len - var_name_len);
+					g_string_truncate (expand, expand->len - var_name_len);
+					g_string_append (expand, value);
+					string = end - 1;
+					continue;
+				}
+				break;
+			}
+			case '~':
+			{
+				/* User home directory expansion */
+				if (isspace(string[1]) || (string[1] == G_DIR_SEPARATOR) || (string[1] == '\0'))
+				{
+					g_string_append (expand, g_get_home_dir());
+					continue;
+				}
+				break;
+			}
+			default:
+				break;
+		}
+		g_string_append_c (expand, *string);
+	}
+	
+	return g_string_free (expand, FALSE);
+}
+
 gchar *
 anjuta_util_str_middle_truncate (const gchar *string,
 								  guint        truncate_length)
@@ -1641,15 +1799,15 @@ anjuta_util_construct_pathv (const gchar* str, va_list str_list)
 	{
 		while ((tmp_str = va_arg (str_list, const gchar*)) != NULL) 
 		{
-			g_ptr_array_add (str_arr, tmp_str);
+			g_ptr_array_add (str_arr, (gpointer)tmp_str);
 		}
 		va_end (str_list);
 	}
 	
 	/* Terminate the list */
-	g_ptr_array_add (str_arr, (gpointer) NULL);
+	g_ptr_array_add (str_arr, NULL);
 
-	path = g_build_filenamev ((gchar*) str_arr->pdata);
+	path = g_build_filenamev ((gchar **)str_arr->pdata);
 	g_ptr_array_free (str_arr, TRUE);
 	
 	return path;
@@ -1778,3 +1936,25 @@ anjuta_util_get_user_data_file_path (const gchar* path, ...)
 	return file_path;
 }
 
+GList *
+anjuta_util_convert_gfile_list_to_path_list (GList *list)
+{
+	GList *path_list;
+	GList *current_file;
+	gchar *path;
+	
+	path_list = NULL;
+	current_file = list;
+	
+	while (current_file)
+	{
+		path = g_file_get_path (current_file->data);
+		
+		/* Ignore files with invalid paths */
+		if (path)
+			path_list = g_list_append (path_list, path);
+	}
+	
+	return path_list;
+}
+
