@@ -2,30 +2,20 @@
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 
 // #include <libgtodo/libgtodo.h>
 #include <libgtodo.h>
 
+#define ICON_FILE "anjuta-gtodo-plugin-48.png"
+
 int  get_all_due_dates();
 xmlNodePtr  get_id_node(gchar *category, gint id);
 extern GtkWidget *window;
 extern GtkListStore *list;
 extern GtkTreeModelSort *sortmodel;
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#include <locale.h>
-#define _(String) gettext (String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else 
-#define _(String) String
-#endif
-
 enum
 {
 	ID,
@@ -61,6 +51,7 @@ typedef struct{
 	GtkTreeModel *sortmodel;
 	GtkWidget *tbdelbut, *tbaddbut, *tbeditbut;
 	GtkWidget *tbeditlb;
+	GtkWidget *tbexport;
 	GtkWidget *option, *menu;
 	catitems **mitems;
 	GtkAccelGroup *accel_group;
@@ -104,6 +95,9 @@ void gtodo_update_settings (void);
 void gtodo_set_hide_done (gboolean hide_it);
 void gtodo_set_hide_due (gboolean hide_it);
 void gtodo_set_hide_nodate (gboolean hide_it);
+gboolean gtodo_get_hide_done (void);
+gboolean gtodo_get_hide_due (void);
+gboolean gtodo_get_hide_nodate (void);
 void gtodo_set_sorting_order (gboolean ascending_sort);
 void gtodo_set_sorting_type (int sorting_type);
 
