@@ -1,9 +1,7 @@
 #include <config.h>
 #include <gtk/gtk.h>
-#include <tray-icon.h>
 #include "main.h"
 
-void preferences_cb_do_tray(GtkWidget *chbox, GtkWidget *hwin);
 static void preferences_cb_show_date(GtkWidget *chbox);
 static void preferences_cb_do_tooltip(GtkWidget *chbox);
 static void preferences_cb_auto_purge(GtkWidget *cb, GtkWidget *hbox);
@@ -13,9 +11,8 @@ static void preferences_cb_toggle_hl_indays(GtkWidget *chbox);
 static void preferences_cb_toggle_show_notification(GtkWidget *chbox);
 static void preferences_cb_toggle_show_category_column(GtkWidget *chbox);
 static void preferences_cb_toggle_show_priority_column(GtkWidget *chbox);
-static void preferences_cb_toggle_enable_tray(GtkWidget *chbox);
 
-void gui_preferences()
+void gui_preferences(void)
 {
 	GtkWidget *dialog;
 	GtkWidget *notebook;
@@ -41,13 +38,41 @@ void gui_preferences()
 	/* save the settings */
 }
 
+static GtkWidget *cb_show_date = NULL;
+static GtkWidget *cb_list_tooltip = NULL;
+static GtkWidget *cb = NULL;
+static GtkWidget *cb_auto_purge = NULL;
+static GtkWidget *sp_purge_days = NULL;
+static GtkWidget *cb_hl_due = NULL;
+static GtkWidget *cb_hl_today = NULL;
+static GtkWidget *cb_hl_indays = NULL;
+static GtkWidget *hbox = NULL;
+
+void preferences_remove_signals()
+{
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb_show_date), 
+		G_CALLBACK(preferences_cb_show_date), NULL);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb),
+		G_CALLBACK(preferences_cb_toggle_show_category_column), NULL);	
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb),
+		G_CALLBACK(preferences_cb_toggle_show_priority_column), NULL);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb_list_tooltip),
+		G_CALLBACK(preferences_cb_do_tooltip), NULL);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb_hl_today), 
+		G_CALLBACK( preferences_cb_toggle_hl_today), NULL);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb_hl_due),
+		G_CALLBACK( preferences_cb_toggle_hl_due), NULL);	
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb_auto_purge),
+		G_CALLBACK(preferences_cb_auto_purge), hbox);
+	g_signal_handlers_disconnect_by_func(G_OBJECT(cb),
+		G_CALLBACK(preferences_cb_toggle_show_notification), NULL);
+}
+			
 GtkWidget *preferences_widget()
 {
 	GtkWidget *vbox,  *vbox2;
-	GtkWidget *frame;
-	GtkWidget *cb_show_date, *cb_list_tooltip, *cb;
-	GtkWidget *cb_auto_purge, *sp_purge_days, *label, *hbox;
-	GtkWidget *cb_hl_due, *cb_hl_today, *cb_hl_indays;
+	GtkWidget *frame, *label;
+	
 	GtkWidget *notebook;
 	gchar *tmp;
 
@@ -91,7 +116,9 @@ GtkWidget *preferences_widget()
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 	frame = gtk_frame_new(_("Show in main window"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);	
-	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), _("<b>Show in main window</b>"));
+	tmp = g_strdup_printf("<b>%s</b>", _("Show in main window"));
+	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), tmp);
+	g_free(tmp);
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
@@ -102,26 +129,31 @@ GtkWidget *preferences_widget()
 	vbox = gtk_vbox_new(FALSE, 0);
 	frame = gtk_frame_new(_("Highlight"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);	
-	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), _("<b>Highlight</b>"));
+	tmp = g_strdup_printf("<b>%s</b>", _("Highlight"));
+	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), tmp);
+	g_free(tmp);
+	
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 	gtk_box_pack_start(GTK_BOX(vbox2), frame, TRUE, TRUE, 0);
 
 	/* tb for highlighting due today */
-	cb_hl_today = 	gtk_check_button_new_with_label(_("items that are due today"));
+	cb_hl_today = 	gtk_check_button_new_with_label(_("Items that are due today"));
 	gtk_box_pack_start(GTK_BOX(vbox),cb_hl_today, FALSE, TRUE, 6); 	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb_hl_today), settings.hl_today);
 	g_signal_connect(G_OBJECT(cb_hl_today),"toggled", G_CALLBACK( preferences_cb_toggle_hl_today), NULL);
 
 	/* tb for highlighting due */
-	cb_hl_due = gtk_check_button_new_with_label(_("items that are past due"));
+	cb_hl_due = gtk_check_button_new_with_label(_("Items that are past due"));
 	gtk_box_pack_start(GTK_BOX(vbox),cb_hl_due, FALSE, TRUE, 6); 	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb_hl_due), settings.hl_due);
 	g_signal_connect(G_OBJECT(cb_hl_due), "toggled",G_CALLBACK( preferences_cb_toggle_hl_due), NULL);	
 
 	/* tb for highlighting in x days */
-	tmp = g_strdup_printf(_("items that are due in the next %i days"), settings.due_days);
+	tmp = g_strdup_printf(ngettext("Items that are due in the next %i day",
+                                 "Items that are due in the next %i days", settings.due_days),
+                                 settings.due_days);
 	cb_hl_indays = gtk_check_button_new_with_label(tmp);
 	g_free(tmp);
 	gtk_box_pack_start(GTK_BOX(vbox),cb_hl_indays, FALSE, TRUE, 6); 	
@@ -140,17 +172,20 @@ GtkWidget *preferences_widget()
 	gtk_box_pack_start(GTK_BOX(vbox),cb_auto_purge, FALSE, TRUE, 0); 	
 
 
-	hbox = gtk_hbox_new(FALSE, 6); 
+	hbox = gtk_hbox_new(FALSE, 6);
+	/* Translators: First part of the sentence "Purge items after %d days"*/
 	label = gtk_label_new(_("Purge items after"));
 	gtk_box_pack_start(GTK_BOX(hbox),label, FALSE, TRUE, 0); 
 	sp_purge_days = gtk_spin_button_new_with_range(1,365, 1);	
 	gtk_box_pack_start(GTK_BOX(hbox),sp_purge_days, FALSE, TRUE, 0); 
+	/* Translators: Second part of the sentence "Purge items after %d days"*/
 	label = gtk_label_new(_("days."));
 	gtk_box_pack_start(GTK_BOX(hbox),label, FALSE, TRUE, 0); 
 
 	gtk_box_pack_start(GTK_BOX(vbox),hbox, FALSE, TRUE, 0); 
 	g_signal_connect(G_OBJECT(cb_auto_purge), "toggled", G_CALLBACK(preferences_cb_auto_purge), hbox);
 
+	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb_auto_purge), settings.auto_purge);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(sp_purge_days), settings.purge_days);
 	/* make sure the sensitive is ok */
@@ -159,7 +194,9 @@ GtkWidget *preferences_widget()
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 	frame = gtk_frame_new(_("Auto Purge"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);	
-	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), _("<b>Auto Purge</b>"));
+	tmp = g_strdup_printf("<b>%s</b>", _("Auto Purge"));
+	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), tmp);
+	g_free(tmp);
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
@@ -171,24 +208,23 @@ GtkWidget *preferences_widget()
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 	frame = gtk_frame_new(_("Notification"));
 	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);	
-	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), _("<b>Notification</b>"));
+	tmp = g_strdup_printf("<b>%s</b>", _("Notification"));
+	gtk_label_set_markup(GTK_LABEL(gtk_frame_get_label_widget(GTK_FRAME(frame))), tmp);
+	g_free(tmp);
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 	gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 0);
-	tmp = g_strdup_printf(_("Allow to do items to notifiy me when they are due in %i minutes"), gconf_client_get_int(client, "/apps/gtodo/prefs/notify_in_minutes",NULL));
+	tmp = g_strdup_printf(ngettext("Allow todo items to notifiy me when they are due in %i minute",
+                                 "Allow todo items to notifiy me when they are due in %i minutes",
+                                 gconf_client_get_int(client, "/apps/gtodo/prefs/notify_in_minutes",NULL)),
+                                 gconf_client_get_int(client, "/apps/gtodo/prefs/notify_in_minutes",NULL));
 	cb = gtk_check_button_new_with_label(tmp);
 	g_free(tmp);
 	gtk_box_pack_start(GTK_BOX(vbox),cb, FALSE, TRUE, 0); 	
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb), gconf_client_get_bool(client, "/apps/gtodo/prefs/do_notification",NULL));
 	g_signal_connect(G_OBJECT(cb), "toggled", G_CALLBACK(preferences_cb_toggle_show_notification), NULL);
 
-	cb = gtk_check_button_new_with_label(_("Show Notification Tray Icon"));
-	gtk_box_pack_start(GTK_BOX(vbox),cb, FALSE, TRUE, 0); 	
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb), 
-		gconf_client_get_bool(client, "/apps/gtodo/view/enable-tray",NULL));
-	g_signal_connect(G_OBJECT(cb), "toggled", 
-			G_CALLBACK(preferences_cb_toggle_enable_tray), NULL);
 	gtk_widget_show_all (notebook);
 	return notebook;
 }
@@ -215,12 +251,6 @@ static void preferences_cb_toggle_hl_indays(GtkWidget *chbox)
 	category_changed();
 }
 
-static void preferences_cb_toggle_enable_tray(GtkWidget *chbox)
-{
-	gconf_client_set_bool(client, "/apps/gtodo/view/enable-tray", 
-			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chbox)),NULL);
-}
-
 static void preferences_cb_toggle_show_notification(GtkWidget *chbox)
 {
 	gconf_client_set_bool(client, "/apps/gtodo/prefs/do_notification", 
@@ -263,16 +293,12 @@ static void preferences_cb_do_tooltip(GtkWidget *chbox)
 	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chbox)))
 
 	{
-		message_box(_(	"Showing Tooltips in the todo list is still very alpha.\n"
-				"Because of some weird behaviour in gtk it only works with the column headers disabled.\n"
-				"I hope to get this fixed soon"),"", GTK_MESSAGE_INFO);
 		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(mw.treeview), FALSE);			
 	}
 	else		
 	{
-
 		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(mw.treeview), TRUE);
-	}		
+	}
 }
 
 static void preferences_cb_auto_purge(GtkWidget *cb, GtkWidget *hbox)
@@ -332,49 +358,12 @@ static void  pref_gconf_changed_auto_purge(GConfClient *client)
 	settings.auto_purge = gconf_client_get_bool(client,"/apps/gtodo/prefs/auto-purge",NULL);
 }    
 
+#if 0
 static void  pref_gconf_changed_auto_purge_days(GConfClient *client)
 {
 	settings.purge_days = gconf_client_get_int(client, "/apps/gtodo/prefs/auto-purge-days",NULL);
 }    
-static void  pref_gconf_changed_sort_type(GConfClient *client)
-{
-	settings.sorttype = gconf_client_get_int(client, "/apps/gtodo/prefs/sort-type",NULL);
-	// FIXME: set_sorting_menu_item();
-}    
-static void  pref_gconf_changed_sort_order(GConfClient *client)
-{
-	settings.sortorder = gconf_client_get_int(client, "/apps/gtodo/prefs/sort-order",NULL);
-	if(settings.sortorder == 0)gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, N_("/View/Sorting/Sort Ascending"))), TRUE);		
-	else gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, N_("/View/Sorting/Sort Descending"))), TRUE);		
-	gtk_tree_sortable_set_sort_column_id((GtkTreeSortable *)mw.sortmodel,0 , settings.sortorder);
-}    
-
-static void  pref_gconf_changed_hide_done(GConfClient *client)
-{
-	settings.hide_done = gconf_client_get_bool(client, "/apps/gtodo/prefs/hide-done",NULL);
-	/*	//    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, _("/View/Todo List/Hide Completed Items"))),settings.hide_done);
-	*/
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, N_("/View/Hide Completed Items"))),settings.hide_done);
-	category_changed();
-}    
-
-static void  pref_gconf_changed_hide_due(GConfClient *client)
-{
-	settings.hide_due = gconf_client_get_bool(client, /* "/schemas*/"/apps/gtodo/prefs/hide-due",NULL);
-	/*//    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, _("/View/Todo List/Hide Items that are past due"))),settings.hide_due);
-	*/
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, N_("/View/Hide Items that Are Past Due Date"))),settings.hide_due);
-	category_changed();
-}
-
-static void  pref_gconf_changed_hide_nodate(GConfClient *client)
-{
-	settings.hide_nodate = gconf_client_get_bool(client, /* "/schemas*/"/apps/gtodo/prefs/hide-nodate",NULL);
-	/*//    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, _("/View/Todo List/Hide Items without an end date"))), settings.hide_nodate);
-	*/
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(mw.item_factory, N_("/View/Hide Items Without an End Date"))), settings.hide_nodate);
-	category_changed();
-}    
+#endif 
 
 static void  pref_gconf_changed_hl_today(GConfClient *client)
 {
@@ -388,19 +377,6 @@ static void  pref_gconf_changed_hl_due(GConfClient *client)
 	category_changed();
 }
 
-static void  pref_gconf_changed_enable_tray(GConfClient *client)
-{
-	if(gconf_client_get_bool(client, "/apps/gtodo/view/enable-tray",NULL))
-	{
-	    if (mw.window)
-		tray_init(mw.window);	
-	}
-	else
-	{
-		tray_icon_remove();
-	}
-}
-
 static void  pref_gconf_changed_hl_indays(GConfClient *client)
 {
 	settings.hl_indays= gconf_client_get_bool(client,"/apps/gtodo/prefs/hl-indays",NULL);
@@ -444,28 +420,7 @@ void pref_gconf_set_notifications(GConfClient *client)
 			(GConfClientNotifyFunc) pref_gconf_changed_auto_purge,
 			NULL,
 			NULL, NULL);
-	gconf_client_notify_add(client,"/apps/gtodo/prefs/hide-done",
-			(GConfClientNotifyFunc) pref_gconf_changed_hide_done,
-			NULL,
-			NULL, NULL);
-	gconf_client_notify_add(client,"/apps/gtodo/prefs/hide-nodate",
-			(GConfClientNotifyFunc) pref_gconf_changed_hide_nodate,
-			NULL,
-			NULL, NULL);
-	gconf_client_notify_add(client,"/apps/gtodo/prefs/hide-due",
-			(GConfClientNotifyFunc) pref_gconf_changed_hide_due,
-			NULL,
-			NULL, NULL);
-	gconf_client_notify_add(client,"/apps/gtodo/prefs/sort-type",
-			(GConfClientNotifyFunc) pref_gconf_changed_sort_type,
-			NULL,
-			NULL, NULL);
-
-	gconf_client_notify_add(client,"/apps/gtodo/prefs/sort-order",
-			(GConfClientNotifyFunc) pref_gconf_changed_sort_order,
-			NULL,
-			NULL, NULL);	
-
+	
 	gconf_client_notify_add(client,"/apps/gtodo/prefs/hl-today",
 			(GConfClientNotifyFunc) pref_gconf_changed_hl_today,
 			NULL,
@@ -480,9 +435,4 @@ void pref_gconf_set_notifications(GConfClient *client)
 			NULL,
 			NULL, NULL);	
 
-	gconf_client_notify_add(client,"/apps/gtodo/view/enable-tray",
-			(GConfClientNotifyFunc) pref_gconf_changed_enable_tray,
-			NULL,
-			NULL, NULL);	
-	
 }
