@@ -24,7 +24,9 @@
 #include <libanjuta/anjuta-debug.h>
 #include <libanjuta/interfaces/ianjuta-iterable.h>
 #include <libanjuta/interfaces/ianjuta-document.h>
+#include <libanjuta/interfaces/ianjuta-document-manager.h>
 #include <libanjuta/interfaces/ianjuta-editor.h>
+#include <libanjuta/interfaces/ianjuta-file.h>
 #include <libanjuta/interfaces/ianjuta-editor-cell.h>
 #include <libanjuta/interfaces/ianjuta-editor-language.h>
 #include <libanjuta/interfaces/ianjuta-editor-selection.h>
@@ -36,9 +38,12 @@
 #include "plugin.h"
 #include "cpp-java-utils.h"
 
+/* Pixmaps */
+#define ANJUTA_PIXMAP_SWAP                "anjuta-swap"
 #define ANJUTA_PIXMAP_COMPLETE			  "anjuta-complete"
 #define ANJUTA_PIXMAP_AUTOCOMPLETE        "anjuta-complete-auto"
 #define ANJUTA_PIXMAP_AUTOINDENT          "anjuta-indent-auto"
+#define ANJUTA_STOCK_SWAP                     "anjuta-swap"
 #define ANJUTA_STOCK_COMPLETE         	  "anjuta-complete"
 #define ANJUTA_STOCK_AUTOCOMPLETE         "anjuta-autocomplete"
 #define ANJUTA_STOCK_AUTOINDENT           "anjuta-indent"
@@ -412,7 +417,7 @@ static void
 set_indentation_param_emacs (CppJavaPlugin* plugin, const gchar *param,
 					   const gchar *value)
 {
-	DEBUG_PRINT ("Setting indent param: %s = %s", param, value);
+	//DEBUG_PRINT ("Setting indent param: %s = %s", param, value);
 	if (strcasecmp (param, "indent-tabs-mode") == 0)
 	{
 		if (strcasecmp (value, "t") == 0)
@@ -444,7 +449,7 @@ static void
 set_indentation_param_vim (CppJavaPlugin* plugin, const gchar *param,
 					   const gchar *value)
 {
-	DEBUG_PRINT ("Setting indent param: %s = %s", param, value);
+	//DEBUG_PRINT ("Setting indent param: %s = %s", param, value);
 	if (g_str_equal (param, "expandtab") ||
 		g_str_equal (param, "et"))
 	{
@@ -693,7 +698,7 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 		gchar *line_string = ianjuta_editor_get_text (editor, line_begin,
 														   line_end, NULL);
 		
-		DEBUG_PRINT ("line_string = '%s'", line_string);
+		//DEBUG_PRINT ("line_string = '%s'", line_string);
 		if (line_string)
 		{
 			idx = line_string;
@@ -713,7 +718,7 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 	*/
 	current_pos = ianjuta_editor_get_position (editor, NULL);
 	carat_offset = ianjuta_iterable_diff (indent_position, current_pos, NULL);
-	DEBUG_PRINT ("carat offset is = %d", carat_offset);
+	//DEBUG_PRINT ("carat offset is = %d", carat_offset);
 	
 	/* Set new indentation */
 	if (indentation > 0)
@@ -731,7 +736,7 @@ set_line_indentation (IAnjutaEditor *editor, gint line_num, gint indentation)
 					ianjuta_editor_get_text (editor, line_begin,
 												  indent_position, NULL);
 				
-				DEBUG_PRINT ("old_indent_string = '%s'", old_indent_string);
+				//DEBUG_PRINT ("old_indent_string = '%s'", old_indent_string);
 				nchars_removed = g_utf8_strlen (old_indent_string, -1);
 			}
 			
@@ -910,9 +915,8 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			} while (ianjuta_iterable_previous (new_iter, NULL));
 			if (comment)
 			{
-				ianjuta_iterable_set_position (iter,
-											   ianjuta_iterable_get_position (new_iter, NULL) - 1, 
-											   NULL);
+				ianjuta_iterable_assign (iter, new_iter, NULL);
+				ianjuta_iterable_previous (iter, NULL);
 				g_object_unref (new_iter);
 				continue;
 			}
@@ -991,9 +995,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 						}
 						if (possible_comment)
 						{
-							ianjuta_iterable_set_position (new_iter, 
-														   ianjuta_iterable_get_position (prev, NULL),
-														   NULL);
+							ianjuta_iterable_assign (new_iter, prev, NULL);
 							g_object_unref (prev);
 							g_object_unref (next);
 							continue;
@@ -1005,9 +1007,8 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			} while (ianjuta_iterable_previous (new_iter, NULL));
 			if (comment)
 			{
-				ianjuta_iterable_set_position (iter,
-											   ianjuta_iterable_get_position (new_iter, NULL) - 1, 
-											   NULL);
+				ianjuta_iterable_assign (iter, new_iter, NULL);
+				ianjuta_iterable_previous (iter, NULL);
 				g_object_unref (new_iter);
 				continue;
 			}
@@ -1043,7 +1044,7 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 				line_indent += extra_indent;
 				break;
 			}
-			}
+		}
 		else if (point_ch == '{')
 		{
 			gint line_for_indent =
@@ -1102,14 +1103,17 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 			/* This is a forward reference, all lines below should have
 			 * increased indentation until the next statement has
 			 * a ':'
+			 * If current line indentation is zero, that we don't indent
 			 */
 			IAnjutaIterable* new_iter = ianjuta_iterable_clone (iter, NULL);
-			gboolean indent = FALSE;		
+			IAnjutaIterable* line_begin;
+			gboolean indent = FALSE;
+			gchar c;
 			
 			/* Is the last non-whitespace in line */
 			while (ianjuta_iterable_next (new_iter, NULL))
 			{
-				gchar c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter),
+				c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (new_iter),
 													   0, NULL);
 				if (!isspace(c))
 					break;
@@ -1119,14 +1123,23 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 					break;
 				}
 			}
-			g_object_unref (new_iter);
+			line_begin = ianjuta_editor_get_line_begin_position(editor,
+																ianjuta_editor_get_line_from_position(editor, iter, NULL),
+																NULL);
+			c = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (line_begin),
+														0, NULL);
 			if (indent)
 			{
 				*colon_indent = TRUE;
-				extra_indent += INDENT_SIZE;
 				if (*incomplete_statement == -1)
 					*incomplete_statement = 0;
 			}
+			if (indent && isspace(c))
+			{
+				extra_indent += INDENT_SIZE;
+			}
+			g_object_unref (new_iter);
+			g_object_unref (line_begin);
 		}	
 		else if (iter_is_newline (iter, point_ch))
 		{
@@ -1159,7 +1172,6 @@ get_line_indentation_base (CppJavaPlugin *plugin,
 	}
 	if (!line_indent && extra_indent)
 	{
-		DEBUG_PRINT ("Adding special indent");
 		line_indent += extra_indent;
 	}
 	g_object_unref (iter);
@@ -1238,9 +1250,9 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 		/* If the last non-whitespace character in the line is ":" then
 		 * we remove the extra colon_indent
 		 */
-		end_iter = ianjuta_editor_get_line_end_position (editor, line, NULL);
 		gchar ch;
-		do
+		end_iter = ianjuta_editor_get_line_end_position (editor, line, NULL);
+		while (ianjuta_iterable_previous (end_iter, NULL))
 		{
 			ch = ianjuta_editor_cell_get_char (IANJUTA_EDITOR_CELL (end_iter),
 											   0, NULL);
@@ -1249,8 +1261,9 @@ get_line_auto_indentation (CppJavaPlugin *plugin, IAnjutaEditor *editor,
 				line_indent -= INDENT_SIZE;
 				break;
 			}
+			if (!isspace(ch) || iter_is_newline (end_iter, ch))
+				break;
 		}
-		while (isspace(ch) && ianjuta_iterable_previous (end_iter, NULL));
 		g_object_unref (end_iter);
 	}
 	
@@ -1334,7 +1347,6 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 	IAnjutaEditorAttribute attrib;
 	IAnjutaIterable *iter;
 	gboolean should_auto_indent = FALSE;
-	static GString *stack = NULL;
 
 	iter = ianjuta_iterable_clone (insert_pos, NULL);
 	
@@ -1388,10 +1400,6 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 				}
 			}
 		}
-		else if (ch == ':')
-		{
-			should_auto_indent = TRUE;
-		}
 		if (should_auto_indent)
 		{
 			gint insert_line;
@@ -1408,34 +1416,50 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 	
 	if (anjuta_preferences_get_int (plugin->prefs, PREF_BRACE_AUTOCOMPLETION))
 	{
-		if (!stack)
-			stack = g_string_new ("");
-	
 		if (ch == '[' || ch == '(')
 		{
-			ianjuta_iterable_next (iter, NULL);
-		
-			switch (ch)
+			gchar *prev_char;
+			IAnjutaIterable *previous;
+			
+			previous = ianjuta_iterable_clone (iter, NULL);
+			ianjuta_iterable_previous (previous, NULL);
+			prev_char = ianjuta_editor_get_text (editor, previous, iter, NULL);
+			
+			/* If the previous char is a ' we don't have to autocomplete */
+			if (*prev_char != '\'')
 			{
-				case '[': ianjuta_editor_insert (editor, iter,
-												 "]", 1, NULL);
-						  g_string_prepend_c (stack, ']');
-						  break;
-				case '(': ianjuta_editor_insert (editor, iter,
-												 ")", 1, NULL);
-						  g_string_prepend_c (stack, ')');
-						  break;
-				default: break;
-			}
+				ianjuta_iterable_next (iter, NULL);
+		
+				switch (ch)
+				{
+					case '[': ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+							  ianjuta_editor_insert (editor, iter,
+													 "]", 1, NULL);
+							  ianjuta_document_end_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+							  break;
+					case '(': ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+							  ianjuta_editor_insert (editor, iter,
+													 ")", 1, NULL);
+							  ianjuta_document_end_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+							  break;
+					default: break;
+				}
 
-			ianjuta_iterable_previous (iter, NULL);
-			ianjuta_editor_goto_position (editor, iter, NULL);
+				ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+				ianjuta_iterable_previous (iter, NULL);
+				ianjuta_editor_goto_position (editor, iter, NULL);
+				ianjuta_document_end_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+			}
+			g_object_unref (previous);
 		}
-		else if (ch == ']' || ch == ')' || ch == '"')
+		else if (ch == ']' || ch == ')' || ch == '"' || ch == '\'')
 		{
-			gchar *str = stack->str;
-			gchar *next_char;
-			IAnjutaIterable *end;
+			gchar *prev_char;
+			IAnjutaIterable *previous;
+			
+			previous = ianjuta_iterable_clone (iter, NULL);
+			ianjuta_iterable_previous (previous, NULL);
+			prev_char = ianjuta_editor_get_text (editor, previous, iter, NULL);
 
 			/* First iter*/
 			ianjuta_iterable_next (iter, NULL);
@@ -1444,34 +1468,33 @@ on_editor_char_inserted_cpp (IAnjutaEditor *editor,
 			 * If the character is " we have to decide if we need insert
 			 * another " or we have to skip the character
 			 */
-			if (ch == '"' && *str != '"')
+			if (ch == '"' || ch == '\'')
 			{
-				ianjuta_editor_insert (editor, iter,
-									 "\"", 1, NULL);
-				g_string_prepend_c (stack, '"');
-			
-				ianjuta_iterable_previous (iter, NULL);
-				ianjuta_editor_goto_position (editor, iter, NULL);
+				/*
+				 * Now we have to detect if we want to manage " as a char
+				 */
+				if (*prev_char != '\'' && *prev_char != '\\')
+				{
+					gchar *c;
+					
+					if (ch == '"')
+						c = g_strdup ("\"");
+					else c = g_strdup ("'");
+					
+					ianjuta_document_begin_undo_action (IANJUTA_DOCUMENT (editor), NULL);
+					ianjuta_editor_insert (editor, iter, c, 1, NULL);
+
+					ianjuta_iterable_previous (iter, NULL);
+					ianjuta_editor_goto_position (editor, iter, NULL);
+					ianjuta_document_end_undo_action (IANJUTA_DOCUMENT (editor), NULL);	
+					
+					g_free (c);
+				}
+				g_object_unref (previous);
 				g_object_unref (iter);
 				return;
 			}
-		
-			/* End iter*/
-			end = ianjuta_iterable_clone (iter, NULL);
-			ianjuta_iterable_next (end, NULL);
-		
-			next_char = ianjuta_editor_get_text (editor, iter, end, NULL);
-		
-			if (ch == *str && ch == *next_char)
-			{
-				g_string_erase (stack, 0, 1);
-				ianjuta_editor_erase (editor, iter, end, NULL);
-			}
-			else {
-				g_string_free (stack, TRUE);
-				stack = g_string_new ("");
-			}
-			g_object_unref (end);
+			g_object_unref (previous);
 		}
 	}
 	g_object_unref (iter);
@@ -1654,6 +1677,117 @@ on_value_removed_current_editor (AnjutaPlugin *plugin, const gchar *name,
 	lang_plugin->current_editor = NULL;
 }
 
+const gchar* SOURCE_EXT[] =
+{
+	".c",
+	".cc",
+	".C",
+	".cpp",
+	".cxx",
+	".ccg",
+	NULL
+};
+
+const gchar* HEADER_EXT[] =
+{
+	".h",
+	".hh",
+	".H",
+	".hpp",
+	".hxx",
+	".hg",
+	NULL
+};
+
+static void
+on_swap_activate (GtkAction* action, gpointer data)
+{
+	GFile* file;
+	GFile* parent;
+	gchar* parent_uri;
+	gchar* basename;
+	gchar* ext;
+	CppJavaPlugin *lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (data);
+	IAnjutaDocumentManager* docman =
+		anjuta_shell_get_interface (ANJUTA_PLUGIN(lang_plugin)->shell,
+									IAnjutaDocumentManager,
+									NULL);
+	if (!lang_plugin->current_editor || !docman)
+		return;
+	
+	file = ianjuta_file_get_file (IANJUTA_FILE (lang_plugin->current_editor),
+								  NULL);
+	parent = g_file_get_parent (file);
+	parent_uri = g_file_get_uri (parent);
+	basename = g_file_get_basename (file);
+	g_object_unref (file);
+	g_object_unref (parent);
+	ext = strstr (basename, ".");
+	if (ext)
+	{
+		int i;
+		for (i = 0; SOURCE_EXT[i] != NULL; i++)
+		{
+			if (g_str_equal (ext, SOURCE_EXT[i]))
+			{
+				int j;
+				for (j = 0; HEADER_EXT[j] != NULL; j++)
+				{
+					gchar* filename;
+					gchar* uri;
+					GFile* new_file;
+					*ext = '\0';
+					filename = g_strdup_printf ("%s%s", basename, HEADER_EXT[j]);
+					uri = g_build_filename (parent_uri, filename, NULL);
+					new_file = g_file_new_for_uri (uri);
+					g_free (uri);
+					g_free(filename);
+					if (g_file_query_exists (new_file, NULL))
+					{
+						ianjuta_document_manager_goto_file_line (docman,
+																 new_file,
+																 -1,
+																 NULL);
+						g_object_unref (new_file);
+						break;
+					}
+					g_object_unref (new_file);
+				}
+				break;
+			}
+			if (g_str_equal (ext, HEADER_EXT[i]))
+			{
+				int j;
+				for (j = 0; SOURCE_EXT[j] != NULL; j++)
+				{
+					gchar* filename;
+					gchar* uri;
+					GFile* new_file;
+					*ext = '\0';
+					filename = g_strdup_printf ("%s%s", basename, SOURCE_EXT[j]);
+					uri = g_build_filename (parent_uri, filename, NULL);
+					new_file = g_file_new_for_uri (uri);
+					g_free (uri);
+					g_free(filename);
+					if (g_file_query_exists (new_file, NULL))
+					{
+						ianjuta_document_manager_goto_file_line (docman,
+																 new_file,
+																 -1,
+																 NULL);
+						g_object_unref (new_file);
+						break;
+					}
+					g_object_unref (new_file);
+				}
+				break;
+			}
+		}
+	}
+	g_free(basename);
+	g_free (parent_uri);
+}
+
 static void
 on_auto_indent (GtkAction *action, gpointer data)
 {
@@ -1727,6 +1861,12 @@ static GtkActionEntry actions[] = {
 		N_("Auto Indent"), "<control>i",
 		N_("Auto indent current line or selection based on indentation settings"),
 		G_CALLBACK (on_auto_indent)
+	},
+	{   "ActionFileSwap", 
+		ANJUTA_STOCK_SWAP, 
+		N_("Swap .h/.c"), NULL,
+		N_("Swap c header and source files"),
+		G_CALLBACK (on_swap_activate)
 	}
 };
 
@@ -1741,6 +1881,7 @@ register_stock_icons (AnjutaPlugin *plugin)
 
 	/* Register stock icons */
 	BEGIN_REGISTER_ICON (plugin);
+	REGISTER_ICON_FULL (ANJUTA_PIXMAP_SWAP, ANJUTA_STOCK_SWAP);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_COMPLETE, ANJUTA_STOCK_COMPLETE);	
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_AUTOCOMPLETE, ANJUTA_STOCK_AUTOCOMPLETE);
 	REGISTER_ICON_FULL (ANJUTA_PIXMAP_AUTOINDENT, ANJUTA_STOCK_AUTOINDENT);
@@ -1756,7 +1897,7 @@ cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 	
 	lang_plugin = ANJUTA_PLUGIN_CPP_JAVA (plugin);
 	
-	DEBUG_PRINT ("AnjutaLanguageCppJavaPlugin: Activating plugin ...");
+	DEBUG_PRINT ("%s", "AnjutaLanguageCppJavaPlugin: Activating plugin ...");
 
 	if (!initialized)
 	{
@@ -1776,7 +1917,7 @@ cpp_java_plugin_activate_plugin (AnjutaPlugin *plugin)
 	
 	lang_plugin->editor_watch_id = 
 		anjuta_plugin_add_watch (plugin,
-								 "document_manager_current_editor",
+								  IANJUTA_DOCUMENT_MANAGER_CURRENT_DOCUMENT,
 								 on_value_added_current_editor,
 								 on_value_removed_current_editor,
 								 plugin);
@@ -1801,7 +1942,7 @@ cpp_java_plugin_deactivate_plugin (AnjutaPlugin *plugin)
 	
 	lang_plugin->action_group = NULL;
 	lang_plugin->uiid = 0;
-	DEBUG_PRINT ("AnjutaLanguageCppJavaPlugin: Deactivated plugin.");
+	DEBUG_PRINT ("%s", "AnjutaLanguageCppJavaPlugin: Deactivated plugin.");
 	return TRUE;
 }
 
