@@ -27,10 +27,11 @@
 
 #include <libanjuta/interfaces/ianjuta-terminal.h>
 #include <libanjuta/interfaces/ianjuta-preferences.h>
+#include <libanjuta/interfaces/ianjuta-project-manager.h>
 
 #include <signal.h>
 
-#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-terminal.ui"
+#define UI_FILE PACKAGE_DATA_DIR"/ui/anjuta-terminal-plugin.ui"
 #define PREFS_GLADE PACKAGE_DATA_DIR"/glade/anjuta-terminal-plugin.glade"
 #define ICON_FILE "anjuta-terminal-plugin-48.png"
 
@@ -59,12 +60,11 @@
 
 #include <gconf/gconf-client.h>
 #include <vte/vte.h>
+#include <vte/reaper.h>
 #include <pwd.h>
 #include <gtk/gtk.h>
 #include <libanjuta/anjuta-plugin.h>
 
-extern char **environ;
-
 extern GType terminal_plugin_get_type (GTypeModule *module);
 #define ANJUTA_PLUGIN_TERMINAL_TYPE         (terminal_plugin_get_type (NULL))
 #define ANJUTA_PLUGIN_TERMINAL(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), ANJUTA_PLUGIN_TERMINAL_TYPE, TerminalPlugin))
@@ -79,17 +79,19 @@ typedef struct _TerminalPluginClass TerminalPluginClass;
 struct _TerminalPlugin{
 	AnjutaPlugin parent;
 	
-	AnjutaUI *ui;
+	gint uiid;
+	GtkActionGroup *action_group;	
+	
 	AnjutaPreferences *prefs;
-	pid_t child_pid;
-	GtkWidget *term;
-	GtkWidget *hbox;
+	GPid child_pid;
+	GtkWidget *shell, *term;
+	GtkWidget *shell_box, *term_box;
 	GtkWidget *frame;
-	GtkWidget *scrollbar;
 	GtkWidget *pref_profile_combo;
 	GtkWidget *pref_default_button;
 	gboolean   widget_added_to_shell;
 	GList *gconf_notify_ids;
+	guint root_watch_id;
 #if OLD_VTE == 1
 	gboolean first_time_realization;
 #endif
@@ -127,65 +129,39 @@ get_profile_key (const gchar *profile, const gchar *key)
 									 NULL);
 
 static void
-preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
+terminal_set_preferences (VteTerminal *term, AnjutaPreferences *pref)
 {
 	GConfClient *client;
 	char *text;
 	int value;
 	gboolean setting;
-	GdkColor color;
-	GtkWidget *vte;
+	GdkColor color[2];
+	GdkColor* foreground;
+	GdkColor* background;
 	gchar *profile;
-	AnjutaPreferences *pref;
-	GSList *profiles;
 	
-	pref = term->prefs;
-	vte = term->term;
 	client = gconf_client_get_default ();
 	
 	g_return_if_fail (client != NULL);
 	
 	/* Update the currently available list of terminal profiles */
-	profiles = gconf_client_get_list (client, GCONF_PROFILE_LIST,
-											  GCONF_VALUE_STRING, NULL);
-	if (profiles)
-	{
-		if (term->pref_profile_combo)
-		{
-			GList *list = NULL;
-			GSList *node = profiles;
-			while (node)
-			{
-				if (node->data)
-					list = g_list_append (list, node->data);
-				node = g_slist_next (node);
-			}
-			gtk_combo_set_popdown_strings (GTK_COMBO (term->pref_profile_combo),
-										   list);
-			g_list_free (list);
-		}
-		g_slist_foreach (profiles, (GFunc)g_free, NULL);
-		g_slist_free (profiles);
-	}
 	setting = anjuta_preferences_get_int (pref,
 										  PREFS_TERMINAL_PROFILE_USE_DEFAULT);
 	if (setting)
 	{
 		/* Use the currently selected profile in gnome-terminal */
 		text = gconf_client_get_string (client, GCONF_DEFAULT_PROFILE, NULL);
-		if (!text)
-			text = g_strdup ("Default");
 	}
 	else
 	{
 		/* Otherwise use the user selected profile */
 		text = anjuta_preferences_get (pref, PREFS_TERMINAL_PROFILE);
-		if (!text)
-			text = g_strdup ("Default");
 	}
+	if (!text || (*text == '\0')) 
+			text = g_strdup ("Default");
 	profile = text;
 	
-	vte_terminal_set_mouse_autohide (VTE_TERMINAL (vte), TRUE);
+	vte_terminal_set_mouse_autohide (term, TRUE);
 
 	/* Set terminal font either using the desktop wide font or g-t one. */
 	setting = GET_PROFILE_BOOL (GCONF_USE_SYSTEM_FONT);
@@ -196,39 +172,38 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	} else {
 		text = GET_PROFILE_STRING (GCONF_VTE_TERMINAL_FONT);
 	}
-	if (text && GTK_WIDGET (vte)->window)
-		vte_terminal_set_font_from_string (VTE_TERMINAL (vte), text);
+	vte_terminal_set_font_from_string (term, text);
 	g_free (text);
 	
 	setting = GET_PROFILE_BOOL (GCONF_CURSOR_BLINK);
-	vte_terminal_set_cursor_blinks (VTE_TERMINAL (vte), setting);
+	vte_terminal_set_cursor_blinks ((term), setting);
 	setting = GET_PROFILE_BOOL (GCONF_SILENT_BELL);
-	vte_terminal_set_audible_bell (VTE_TERMINAL (vte), !setting);
+	vte_terminal_set_audible_bell (term, !setting);
 	value = GET_PROFILE_INT (GCONF_SCROLLBACK_LINES);
-	vte_terminal_set_scrollback_lines (VTE_TERMINAL (vte), value);
+	vte_terminal_set_scrollback_lines (term, value);
 	setting = GET_PROFILE_BOOL (GCONF_SCROLL_ON_KEYSTROKE);
-	vte_terminal_set_scroll_on_keystroke (VTE_TERMINAL (vte), setting);
+	vte_terminal_set_scroll_on_keystroke (term, setting);
 	setting = GET_PROFILE_BOOL (GCONF_SCROLL_ON_OUTPUT);
-	vte_terminal_set_scroll_on_output (VTE_TERMINAL (vte), TRUE);
+	vte_terminal_set_scroll_on_output (term, TRUE);
 	text = GET_PROFILE_STRING (GCONF_WORD_CHARS);
 	if (text)
-		vte_terminal_set_word_chars (VTE_TERMINAL (vte), text);
+		vte_terminal_set_word_chars (term, text);
 	g_free (text);
 	
 	text = GET_PROFILE_STRING (GCONF_BACKSPACE_BINDING);
 	if (text)
 	{
 		if (!strcmp (text, "ascii-del"))
-			vte_terminal_set_backspace_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_backspace_binding (term,
 								VTE_ERASE_ASCII_DELETE);
 		else if (!strcmp (text, "escape-sequence"))
-			vte_terminal_set_backspace_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_backspace_binding (term,
 								VTE_ERASE_DELETE_SEQUENCE);
 		else if (!strcmp (text, "control-h"))
-			vte_terminal_set_backspace_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_backspace_binding (term,
 								VTE_ERASE_ASCII_BACKSPACE);
 		else
-			vte_terminal_set_backspace_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_backspace_binding (term,
 								VTE_ERASE_AUTO);
 		g_free (text);
 	}
@@ -236,16 +211,16 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	if (text)
 	{
 		if (!strcmp (text, "ascii-del"))
-			vte_terminal_set_delete_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_delete_binding (term,
 							 VTE_ERASE_ASCII_DELETE);
 		else if (!strcmp (text, "escape-sequence"))
-			vte_terminal_set_delete_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_delete_binding (term,
 							 VTE_ERASE_DELETE_SEQUENCE);
 		else if (!strcmp (text, "control-h"))
-			vte_terminal_set_delete_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_delete_binding (term,
 							 VTE_ERASE_ASCII_BACKSPACE);
 		else
-			vte_terminal_set_delete_binding (VTE_TERMINAL (vte),
+			vte_terminal_set_delete_binding (term,
 							 VTE_ERASE_AUTO);
 		g_free (text);
 	}
@@ -253,22 +228,32 @@ preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
 	text = GET_PROFILE_STRING (GCONF_BACKGROUND_COLOR);
 	if (text)
 	{
-		gdk_color_parse (text, &color);
-		vte_terminal_set_color_background (VTE_TERMINAL (vte), &color);
+		gdk_color_parse (text, &color[0]);
 		g_free (text);
 	}
+	background = text ? &color[0] : NULL;
 	text = GET_PROFILE_STRING (GCONF_FOREGROUND_COLOR);
 	if (text)
 	{
-		gdk_color_parse (text, &color);
-		vte_terminal_set_color_foreground (VTE_TERMINAL (vte), &color);
-		vte_terminal_set_color_bold (VTE_TERMINAL (vte), &color);
+		gdk_color_parse (text, &color[1]);
 		g_free (text);
 	}
+	foreground = text ? &color[1] : NULL;
+	/* vte_terminal_set_colors works even if the terminal widget is not realized
+	 * which is not the case with vte_terminal_set_color_foreground and
+	 * vte_terminal_set_color_background */
+	vte_terminal_set_colors (term, foreground, background, NULL, 0);
 	g_free (profile);
 	g_object_unref (client);
 }
 
+static void
+preferences_changed (AnjutaPreferences *prefs, TerminalPlugin *term)
+{
+	terminal_set_preferences (VTE_TERMINAL (term->shell), prefs);
+	terminal_set_preferences (VTE_TERMINAL (term->term), prefs);
+}
+
 static void
 on_gconf_notify_prefs (GConfClient *gclient, guint cnxn_id,
 					   GConfEntry *entry, gpointer user_data)
@@ -315,50 +300,36 @@ use_default_profile_cb (GtkToggleButton *button,
 		gtk_widget_set_sensitive (term->pref_profile_combo, TRUE);
 }
 
-static char **
-get_child_environment (void)
-{
-	/* code from gnome-terminal, sort of. */
-	char **p;
-	int i;
-	char **retval;
-#define EXTRA_ENV_VARS 6
-
-	/* count env vars that are set */
-	for (p = environ; *p; p++);
-
-	i = p - environ;
-	retval = g_new (char *, i + 1 + EXTRA_ENV_VARS);
-
-	for (i = 0, p = environ; *p; p++) {
-		/* Strip all these out, we'll replace some of them */
-		if ((strncmp (*p, "COLUMNS=", 8) == 0) ||
-		    (strncmp (*p, "LINES=", 6) == 0)   ||
-		    (strncmp (*p, "TERM=", 5) == 0)    ||
-		    (strncmp (*p, "GNOME_DESKTOP_ICON=", 19) == 0)) {
-			/* nothing: do not copy */
-		} else {
-			retval[i] = g_strdup (*p);
-			++i;
-		}
+static void
+terminal_child_exited_cb (VteReaper *reaper, GPid pid, gint status, TerminalPlugin *term_plugin)
+{	
+	gboolean focus;
+	
+	if (term_plugin->child_pid == pid)
+	{
+		focus = gtk_widget_is_focus (term_plugin->term);
+		
+		gtk_container_remove (GTK_CONTAINER (term_plugin->frame), term_plugin->term_box);
+		gtk_container_add (GTK_CONTAINER (term_plugin->frame), term_plugin->shell_box);
+		gtk_widget_show_all (term_plugin->shell_box);
+		if (focus)
+			gtk_widget_grab_focus (term_plugin->shell);
+		
+		term_plugin->child_pid = 0;
 	}
-
-	retval[i] = g_strdup ("TERM=xterm"); /* FIXME configurable later? */
-	++i;
-
-	retval[i] = NULL;
-
-	return retval;
+	
+	g_signal_emit_by_name(term_plugin, "child-exited", pid, status);
 }
 
 static pid_t
 terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
-				  const gchar *command)
+				  const gchar *command, gchar **environment)
 {
-	char **env, **args, **args_ptr;
+	char **args, **args_ptr;
 	GList *args_list, *args_list_ptr;
 	gchar *dir;
 	VteTerminal *term;
+	gboolean focus;
 	
 	g_return_val_if_fail (command != NULL, 0);
 	
@@ -383,19 +354,24 @@ terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
 	term = VTE_TERMINAL (term_plugin->term);
 	
 	vte_terminal_reset (term, TRUE, TRUE);
-
-	env = get_child_environment ();
 	
+	focus = gtk_widget_is_focus (term_plugin->shell);
+	
+	gtk_container_remove (GTK_CONTAINER (term_plugin->frame), term_plugin->shell_box);
+	gtk_container_add (GTK_CONTAINER (term_plugin->frame), term_plugin->term_box);
+	gtk_widget_show_all (term_plugin->term_box);
+	if (focus)
+		gtk_widget_grab_focus (term_plugin->term);
+		
 	term_plugin->child_pid = vte_terminal_fork_command (term, args[0], args,
-														env, dir, 0, 0, 0);
+														environment, dir, 0, 0, 0);
+	vte_reaper_add_child (term_plugin->child_pid);
+	
 	g_free (dir);
-	g_strfreev (env);
 	g_free (args);
 	g_list_foreach (args_list, (GFunc)g_free, NULL);
 	g_list_free (args_list);
 	
-	preferences_changed (term_plugin->prefs, term_plugin);
-	
 	if (term_plugin->widget_added_to_shell)
 		anjuta_shell_present_widget (ANJUTA_PLUGIN (term_plugin)->shell,
 									 term_plugin->frame, NULL);
@@ -404,11 +380,12 @@ terminal_execute (TerminalPlugin *term_plugin, const gchar *directory,
 }
 
 static void
-terminal_init_cb (GtkWidget *widget, TerminalPlugin *term_plugin)
+init_shell (VteTerminal *term, const char *uri)
 {
 	struct passwd *pw;
 	const char *shell;
 	const char *dir;
+	static gboolean first_time = TRUE;
 	
 	pw = getpwuid (getuid ());
 	if (pw) {
@@ -418,52 +395,69 @@ terminal_init_cb (GtkWidget *widget, TerminalPlugin *term_plugin)
 		shell = "/bin/sh";
 		dir = "/";
 	}
-	terminal_execute (term_plugin, dir, shell);
+	
+	if (uri)
+		dir = uri;
+	
+	if (!first_time)
+		vte_terminal_reset (term, FALSE, TRUE);
+	else
+		first_time = FALSE;
+	
+	vte_terminal_fork_command (term, shell, NULL, NULL, dir, 0, 0, 0);
 }
 
 static gboolean
 terminal_focus_cb (GtkWidget *widget, GdkEvent  *event,
 				   TerminalPlugin *term) 
 {
-	if (term->child_pid == 0)
-	{
-		terminal_init_cb (widget, term);
-	}
 	gtk_widget_grab_focus (widget);
 	return FALSE;
 }
 
 static gboolean
 terminal_keypress_cb (GtkWidget *widget, GdkEventKey  *event,
-					  TerminalPlugin *term) 
+						   TerminalPlugin *terminal_plugin)
 {
-	/* Fixme: GDK_KEY_PRESS doesn't seem to be called for our keys */
-	if (event->type != GDK_KEY_RELEASE)
+	if (event->type != GDK_KEY_PRESS)
 		return FALSE;
 	
 	/* ctrl-d */
-	if (event->keyval == GDK_d ||
-		event->keyval == GDK_D)
+	if ((event->keyval == GDK_d || event->keyval == GDK_D) &&
+		(event->state & GDK_CONTROL_MASK))
 	{
-		/* Ctrl pressed */
-		if (event->state & GDK_CONTROL_MASK)
+		if (terminal_plugin->child_pid)
 		{
-			kill (term->child_pid, SIGINT);
-			term->child_pid = 0;
-			terminal_init_cb (GTK_WIDGET (term->term), term);
-			return TRUE;
+			kill (terminal_plugin->child_pid, SIGINT);
+			terminal_plugin->child_pid = 0;
 		}
-	}
-	/* Shift-Insert */
-	if ((event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert) &&
-		 event->state & GDK_SHIFT_MASK)
-	{
-		vte_terminal_paste_clipboard(VTE_TERMINAL(term->term));
 		return TRUE;
 	}
 	return FALSE;
 }
 
+static gboolean
+terminal_click_cb (GtkWidget *widget, GdkEventButton *event,
+				   TerminalPlugin *term) 
+{
+	if (event->button == 3)
+	{
+		AnjutaUI *ui;
+		GtkMenu *popup;
+		GtkAction *action;
+
+		ui = anjuta_shell_get_ui (ANJUTA_PLUGIN(term)->shell, NULL);
+		popup =  GTK_MENU (gtk_ui_manager_get_widget (GTK_UI_MANAGER (ui), "/PopupTerminal"));
+		action = gtk_action_group_get_action (term->action_group, "ActionCopyFromTerminal");
+		gtk_action_set_sensitive (action,vte_terminal_get_has_selection(VTE_TERMINAL(widget)));
+		
+		gtk_menu_popup (popup, NULL, NULL, NULL, NULL,
+						event->button, event->time);
+	}
+	
+	return FALSE;
+}
+
 #if OLD_VTE == 1
 /* VTE has a terrible bug where it could crash when container is changed.
  * The problem has been traced in vte where its style-set handler does not
@@ -510,58 +504,152 @@ terminal_unrealize_cb (GtkWidget *term, TerminalPlugin *plugin)
 #endif
 
 static void
-terminal_destroy_cb (GtkWidget *widget, TerminalPlugin *term)
+on_terminal_copy_cb (GtkAction * action, TerminalPlugin *term_plugin)
 {
-	g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
-							G_CALLBACK (terminal_init_cb), term);
+	VteTerminal *term;
+	
+	if (term_plugin->child_pid)
+		term = VTE_TERMINAL (term_plugin->term);
+	else
+		term = VTE_TERMINAL (term_plugin->shell);
+	
+	if (vte_terminal_get_has_selection(term))
+		vte_terminal_copy_clipboard(term);
 }
 
+static void
+on_terminal_paste_cb (GtkAction * action, TerminalPlugin *term_plugin)
+{
+	VteTerminal *term;
+	
+	if (term_plugin->child_pid)
+		term = VTE_TERMINAL (term_plugin->term);
+	else
+		term = VTE_TERMINAL (term_plugin->shell);
+	
+	vte_terminal_paste_clipboard(term);
+}
+
+static GtkActionEntry actions_terminal[] = {
+	{
+		"ActionCopyFromTerminal", 	              /* Action name */
+		GTK_STOCK_COPY,                           /* Stock icon, if any */
+		N_("_Copy"),		                      /* Display label */
+		NULL,                                     /* short-cut */
+		NULL,                                     /* Tooltip */
+		G_CALLBACK (on_terminal_copy_cb)          /* action callback */
+	},
+	{
+		"ActionPasteInTerminal",
+		GTK_STOCK_PASTE,
+		N_("_Paste"),
+		NULL,
+		NULL,
+		G_CALLBACK (on_terminal_paste_cb)
+	}
+};
+
+static void
+on_project_root_added (AnjutaPlugin *plugin, const gchar *name,
+					   const GValue *value, gpointer user_data)
+{
+	TerminalPlugin *term_plugin;
+	const gchar *root_uri;
+	
+	term_plugin = (TerminalPlugin *)plugin;
+	
+	root_uri = g_value_get_string (value);
+	if (root_uri)
+	{
+		GFile *file;
+		char *path;
+		
+		file = g_file_new_for_uri (root_uri);
+		path = g_file_get_path (file);
+		
+		init_shell (VTE_TERMINAL (term_plugin->shell), path);
+		
+		g_object_unref (file);
+		g_free (path);
+	}
+}
+
+static GtkWidget *
+create_terminal (TerminalPlugin *term_plugin)
+{
+	GtkWidget *term;
+	
+	/* Create new terminal. */
+	term = vte_terminal_new ();
+	gtk_widget_set_size_request (GTK_WIDGET (term), 10, 10);
+	vte_terminal_set_size (VTE_TERMINAL (term), 50, 1);
+	
+	g_signal_connect (G_OBJECT (term), "focus-in-event",
+					  G_CALLBACK (terminal_focus_cb), term_plugin);
+	
+	g_signal_connect (G_OBJECT (term), "button-press-event",
+					  G_CALLBACK (terminal_click_cb), term_plugin);
+	
+#if OLD_VTE == 1
+	g_signal_connect (G_OBJECT (term), "realize",
+					  G_CALLBACK (terminal_realize_cb), term_plugin);
+	g_signal_connect (G_OBJECT (term), "unrealize",
+					  G_CALLBACK (terminal_unrealize_cb), term_plugin);
+#endif
+	
+	return term;
+}
+
+static GtkWidget *
+create_box (GtkWidget *term)
+{
+	GtkWidget *sb, *hbox;
+	
+	sb = gtk_vscrollbar_new (GTK_ADJUSTMENT (VTE_TERMINAL (term)->adjustment));
+	GTK_WIDGET_UNSET_FLAGS (sb, GTK_CAN_FOCUS);
+	
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), term, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, TRUE, 0);
+	
+	g_object_ref_sink (hbox);
+	
+	return hbox;
+}	
+	
 static void
 terminal_create (TerminalPlugin *term_plugin)
 {
-	GtkWidget *sb, *frame, *hbox;
+	GtkWidget *frame;
 	
 	g_return_if_fail(term_plugin != NULL);
 
 	term_plugin->child_pid = 0;
 	
-	/* Create new terminal. */
-	term_plugin->term = vte_terminal_new ();
-	gtk_widget_set_size_request (GTK_WIDGET (term_plugin->term), 10, 10);
-	vte_terminal_set_size (VTE_TERMINAL (term_plugin->term), 50, 1);
+	/* Create the terminals. */
+	term_plugin->shell = create_terminal (term_plugin);
+	term_plugin->shell_box = create_box (term_plugin->shell);
 	
-	g_signal_connect (G_OBJECT (term_plugin->term), "focus_in_event",
-					  G_CALLBACK (terminal_focus_cb), term_plugin);
-	g_signal_connect (G_OBJECT (term_plugin->term), "child-exited",
-					  G_CALLBACK (terminal_init_cb), term_plugin);
-	g_signal_connect (G_OBJECT (term_plugin->term), "destroy",
-					  G_CALLBACK (terminal_destroy_cb), term_plugin);
-	g_signal_connect (G_OBJECT (term_plugin->term), "event",
+	term_plugin->term = create_terminal (term_plugin);
+	term_plugin->term_box = create_box (term_plugin->term);
+	
+	/* key-press handler for ctrl-d "kill" */
+	g_signal_connect (G_OBJECT (term_plugin->term), "key-press-event",
 					  G_CALLBACK (terminal_keypress_cb), term_plugin);
-#if OLD_VTE == 1
-	g_signal_connect (G_OBJECT (term_plugin->term), "realize",
-					  G_CALLBACK (terminal_realize_cb), term_plugin);
-	g_signal_connect (G_OBJECT (term_plugin->term), "unrealize",
-					  G_CALLBACK (terminal_unrealize_cb), term_plugin);
-#endif
-	sb = gtk_vscrollbar_new (GTK_ADJUSTMENT (VTE_TERMINAL (term_plugin->term)->adjustment));
-	GTK_WIDGET_UNSET_FLAGS (sb, GTK_CAN_FOCUS);
 	
 	frame = gtk_frame_new (NULL);
 	gtk_widget_show (frame);
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (frame), hbox);
-	gtk_box_pack_start (GTK_BOX (hbox), term_plugin->term, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), term_plugin->shell_box);
 	gtk_widget_show_all (frame);
 	
-	term_plugin->scrollbar = sb;
 	term_plugin->frame = frame;
-	term_plugin->hbox = hbox;
-
-	terminal_init_cb (GTK_WIDGET (term_plugin->term), term_plugin);
+	
+	g_signal_connect (vte_reaper_get(), "child-exited",
+					  G_CALLBACK (terminal_child_exited_cb), term_plugin);
+	
+	init_shell (VTE_TERMINAL (term_plugin->shell), NULL);
 }
 
 static void
@@ -583,13 +671,22 @@ activate_plugin (AnjutaPlugin *plugin)
 {
 	TerminalPlugin *term_plugin;
 	static gboolean initialized = FALSE;
+	AnjutaUI *ui;
 	
-	DEBUG_PRINT ("TerminalPlugin: Activating Terminal plugin ...");
+	DEBUG_PRINT ("%s", "TerminalPlugin: Activating Terminal plugin ...");
 	
 	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
-	term_plugin->ui = anjuta_shell_get_ui (plugin->shell, NULL);
 	term_plugin->prefs = anjuta_shell_get_preferences (plugin->shell, NULL);
 	term_plugin->widget_added_to_shell = FALSE;
+	ui = anjuta_shell_get_ui (plugin->shell, NULL);
+	term_plugin->action_group = anjuta_ui_add_action_group_entries (ui,
+										"ActionGroupTerminal",
+										_("terminal operations"),
+										actions_terminal,
+										G_N_ELEMENTS (actions_terminal),
+										GETTEXT_PACKAGE, TRUE, term_plugin);
+	term_plugin->uiid = anjuta_ui_merge (ui, UI_FILE);
+	
 	terminal_create (term_plugin);
 	
 	if (!initialized)
@@ -608,6 +705,18 @@ activate_plugin (AnjutaPlugin *plugin)
 	/* terminal_focus_cb (term_plugin->term, NULL, term_plugin); */
 	term_plugin->widget_added_to_shell = TRUE;
 	initialized = TRUE;
+
+	/* Set all terminal preferences, at that time the terminal widget is
+	 * not realized, a few vte functions are not working. Another
+	 * possibility could be to call this when the widget is realized */
+	preferences_changed (term_plugin->prefs, term_plugin);
+	
+	/* set up project directory watch */
+	term_plugin->root_watch_id = anjuta_plugin_add_watch (plugin,
+														  IANJUTA_PROJECT_MANAGER_PROJECT_ROOT_URI,
+														  on_project_root_added,
+														  0, NULL);
+	
 	return TRUE;
 }
 
@@ -615,55 +724,66 @@ static gboolean
 deactivate_plugin (AnjutaPlugin *plugin)
 {
 	TerminalPlugin *term_plugin;
+	AnjutaUI *ui;	
+	
 	term_plugin = ANJUTA_PLUGIN_TERMINAL (plugin);
+
+	ui = anjuta_shell_get_ui (plugin->shell, NULL);
+	anjuta_ui_unmerge (ui, term_plugin->uiid);	
+	if (term_plugin->action_group)
+	{
+		anjuta_ui_remove_action_group (ui, term_plugin->action_group);
+		term_plugin->action_group = NULL;
+	}
 	
 	prefs_finalize (term_plugin);
 
-#if OLD_VTE == 1
-	g_signal_handlers_disconnect_by_func (G_OBJECT (term_plugin->term),
-			 G_CALLBACK (terminal_unrealize_cb), term_plugin);
-#endif
-
 	/* terminal plugin widgets are destroyed as soon as it is removed */
 	anjuta_shell_remove_widget (plugin->shell, term_plugin->frame, NULL);
 	
-	/*
-	g_signal_handlers_disconnect_by_func (G_OBJECT (term_plugin->pref_default_button),
-										  G_CALLBACK (use_default_profile_cb),
+	g_object_unref (term_plugin->shell_box);
+	g_object_unref (term_plugin->term_box);
+	
+	g_signal_handlers_disconnect_by_func (vte_reaper_get (), terminal_child_exited_cb,
 										  term_plugin);
-	*/
-	term_plugin->frame = NULL;
-	term_plugin->term = NULL;
-	term_plugin->scrollbar = NULL;
-	term_plugin->hbox = NULL;
+	
+	/* remove watch */
+	anjuta_plugin_remove_watch (plugin, term_plugin->root_watch_id, FALSE);
+	
 	term_plugin->child_pid = 0;
+	
 #if OLD_VTE == 1
+	g_signal_handlers_disconnect_by_func (G_OBJECT (term_plugin->term),
+			 G_CALLBACK (terminal_unrealize_cb), term_plugin);
+	
 	term_plugin->first_time_realization = TRUE;
 #endif
 
-	// terminal_finalize (term_plugin);
 	return TRUE;
 }
 
 static void
 terminal_plugin_dispose (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, dispose, (obj));
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
 terminal_plugin_finalize (GObject *obj)
 {
-	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (obj));
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 static void
 terminal_plugin_instance_init (GObject *obj)
 {
-	TerminalPlugin *plugin = ANJUTA_PLUGIN_TERMINAL (obj);
-	plugin->gconf_notify_ids = NULL;
-	plugin->child_pid = 0;
-	plugin->pref_profile_combo = NULL;
+	TerminalPlugin *term_plugin = ANJUTA_PLUGIN_TERMINAL (obj);
+	
+	term_plugin->gconf_notify_ids = NULL;
+	term_plugin->child_pid = 0;
+	term_plugin->pref_profile_combo = NULL;
+	term_plugin->uiid = 0;
+	term_plugin->action_group = NULL;
 #if OLD_VTE == 1
 	plugin->first_time_realization = TRUE;
 #endif
@@ -685,7 +805,8 @@ terminal_plugin_class_init (GObjectClass *klass)
 static pid_t
 iterminal_execute_command (IAnjutaTerminal *terminal,
 						   const gchar *directory,
-						   const gchar *command, GError **err)
+						   const gchar *command,
+						   gchar **environment, GError **err)
 {
 	TerminalPlugin *plugin;
 	const gchar *dir;
@@ -697,7 +818,7 @@ iterminal_execute_command (IAnjutaTerminal *terminal,
 	else
 		dir = directory;
 	
-	return terminal_execute (plugin, directory, command);
+	return terminal_execute (plugin, directory, command, environment);
 }
 
 static void
@@ -706,20 +827,80 @@ iterminal_iface_init(IAnjutaTerminalIface *iface)
 	iface->execute_command = iterminal_execute_command;
 }
 
+static void
+on_add_string_in_store (gpointer data, gpointer user_data)
+{
+	GtkListStore* model = (GtkListStore *)user_data;
+	GtkTreeIter iter;
+	
+	gtk_list_store_append (model, &iter);
+	gtk_list_store_set (model, &iter, 0, (const gchar *)data, -1);
+}
+
+static void
+on_concat_string (gpointer data, gpointer user_data)
+{
+	GString* str = (GString *)user_data;	
+	
+	if (str->len != 0)
+		g_string_append_c (str, ',');
+	g_string_append (str, data);
+}
+
 static void
 ipreferences_merge(IAnjutaPreferences* ipref, AnjutaPreferences* prefs, GError** e)
 {
+	GSList *profiles;
+	GConfClient *client;	
+	
 	/* Create the terminal preferences page */
 	TerminalPlugin* term_plugin = ANJUTA_PLUGIN_TERMINAL (ipref);
 	GladeXML *gxml = glade_xml_new (PREFS_GLADE, "preferences_dialog_terminal", NULL);
 	anjuta_preferences_add_page (term_plugin->prefs, gxml,
 									"Terminal", _("Terminal"), ICON_FILE);
+	
 	term_plugin->pref_profile_combo = glade_xml_get_widget (gxml, "profile_list_combo");
-	term_plugin->pref_default_button =
-		glade_xml_get_widget (gxml,
-						"preferences_toggle:bool:1:0:terminal.default.profile");
-	g_signal_connect (G_OBJECT(term_plugin->pref_default_button), "toggled",
+	term_plugin->pref_default_button = glade_xml_get_widget (gxml, "preferences_toggle:bool:1:0:terminal.default.profile");
+
+	/* Update the currently available list of terminal profiles */
+	client = gconf_client_get_default ();
+	profiles = gconf_client_get_list (client, GCONF_PROFILE_LIST,
+									  GCONF_VALUE_STRING, NULL);
+	if (profiles)
+	{
+		GtkListStore *store;
+		GString *default_value;
+			
+		default_value = g_string_new (NULL);
+		store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (term_plugin->pref_profile_combo)));
+		
+		gtk_list_store_clear (store);
+		g_slist_foreach (profiles, on_add_string_in_store, store);
+		g_slist_foreach (profiles, on_concat_string, default_value);
+		g_slist_foreach (profiles, (GFunc)g_free, NULL);
+		g_slist_free (profiles);
+		
+		anjuta_preferences_register_property_raw (term_plugin->prefs,
+												  term_plugin->pref_profile_combo,
+												  PREFS_TERMINAL_PROFILE,
+												  default_value->str,
+												  1,
+												  ANJUTA_PROPERTY_OBJECT_TYPE_COMBO,
+												  ANJUTA_PROPERTY_DATA_TYPE_TEXT);
+		g_string_free (default_value, TRUE);
+		
+		use_default_profile_cb (GTK_TOGGLE_BUTTON (term_plugin->pref_default_button), term_plugin);
+		g_signal_connect (G_OBJECT(term_plugin->pref_default_button), "toggled",
 						  G_CALLBACK (use_default_profile_cb), term_plugin);
+	}
+	else
+	{
+		/* No profile, perhaps GNOME Terminal is not installed,
+		 * Remove selection */
+		gtk_widget_set_sensitive (term_plugin->pref_profile_combo, FALSE);
+		gtk_widget_set_sensitive (term_plugin->pref_default_button, FALSE);
+	}
+	
 	g_object_unref (gxml);
 }
 
